package sky.itlum;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.reflect.Field;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.ShortMessage;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import sky.monome.Monome;
import sky.monome.Monome.MonomeSize;
import sky.monome.MultiSequencerGroup;
import sky.monome.MultiSequencerGroup.SpecificSequencerGroup;
import sky.monome.behavior.OptionalRadio.OptionalRadioGroup;
import sky.monome.event.button.ButtonEvent;
import sky.monome.event.button.ButtonListenerWithArguments;
import sky.monome.event.sequencer.NotePlayerListener;
import sky.monome.event.sequencer.NotePlayerListener.PlayerMode;
import sky.monome.exception.MonomeException;
import sky.monome.sequence.BackwardSequenceIterator;
import sky.monome.sequence.RandomOffsetSequenceIterator;
import sky.monome.sequence.RandomSequenceIterator;
import sky.monome.sequence.SequenceIterator;
import sky.monome.sequence.SwingSequenceIterator;
import sky.monome.sequence.TwiceUpwardSequenceIterator;
import sky.monome.sequence.UpwardAndBackwardSequenceIterator;
import sky.monome.sequence.UpwardSequenceIterator;
import sky.monome.sequence.factory.GenericSequenceIteratorFactory;
import sky.monome.sequencer.Sequencer.SequencerOrientation;
import sky.monome.util.scale.MidiNote;
import sky.monome.util.scale.MidiNote.Note;
import sky.monome.util.scale.Scale;
import sky.monome.util.scale.ScaleFactory;

/**
 * Itlum : the multi example.
 * @author PJ Skyman
 */
public class Itlum extends JFrame
{
    private MidiDevice midiDevice;
    private boolean midiReady;
    private final MultiSequencerGroup multiSequencerGroup;
    private final NotePlayerListener<SpecificSequencerGroup,OptionalRadioGroup>[] notePlayerListeners;
    private final Object lock;

    @SuppressWarnings("unchecked")
    public Itlum() throws MonomeException
    {
        //Preparing the IHM
        System.out.println("Preparing the IHM");
        initComponents();
        lock=new Object();
        Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
        Dimension window=getSize();
        setBounds(screen.width/2-window.width/2,screen.height/2-window.height/2,window.width,window.height);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setTitle("Itlum v1.2 (Demo for SkyMonome v1.2)");
        addWindowListener(new WindowAdapter()
        {
            @Override
            public void windowClosing(WindowEvent e)
            {
                if(midiDevice!=null)
                    midiDevice.close();
                System.exit(0);
            }
        });

        //Building the Monome tree
        System.out.println("Building the Monome tree");
        Monome monome=new Monome("Monome",MonomeSize.MONOME_64,"localhost","/40h",8000,8080);//Creation of the logical Monome device
        multiSequencerGroup=new MultiSequencerGroup("MultiSequencerGroup",0,0,8,8,SequencerOrientation.X_FOR_ANYTHING_Y_FOR_TIME,124f,2,GenericSequenceIteratorFactory.<SpecificSequencerGroup,OptionalRadioGroup>createGenericSequenceIteratorFactory(UpwardSequenceIterator.class));//Creation of the multisequencergroup
        monome.addComponent(multiSequencerGroup);//Adding of the multisequencergroup in the logical Monome device
        notePlayerListeners=new NotePlayerListener[8];
        for(int i=0;i<8;i++)//For each internal sequencer of the multisequencergroup
        {
            SpecificSequencerGroup specificSequencerGroup=multiSequencerGroup.getSequencer(i);
            OptionalRadioGroup optionalRadioGroup=specificSequencerGroup.getButtonManager(0);//We take the unique radio group
            optionalRadioGroup.addButtonListener(new ButtonListenerWithArguments<OptionalRadioGroup>(specificSequencerGroup)//And we add a button listener that will set the number of steps of the current internal sequencer
            {
                public void buttonActionned(ButtonEvent<OptionalRadioGroup> buttonEvent)
                {
                    if(buttonEvent.getSource2().getActiveRadio()!=null)
                        try
                        {
                            this.<SpecificSequencerGroup>getArgument(0).setStepsNumber(8-buttonEvent.getSource2().getActiveRadio().getLedButtonCouple().getY());
                        }
                        catch(MonomeException e)
                        {
                            e.printStackTrace();
                        }
                }
            });
            optionalRadioGroup.addButtonListener(new ButtonListenerWithArguments<OptionalRadioGroup>(i,optionalRadioGroup)//Then we add a button listener that will update the IHM
            {
                public void buttonActionned(ButtonEvent<OptionalRadioGroup> buttonEvent)
                {
                    try
                    {
                        OptionalRadioGroup optionalRadioGroup=this.<OptionalRadioGroup>getArgument(1);
                        Field checkBoxField=Itlum.class.getField("checkBox"+(getArgument(int.class,0)+1));
                        Field sliderField=Itlum.class.getField("slider"+(getArgument(int.class,0)+1));
                        if(optionalRadioGroup.getActiveRadio()==null)
                            ((JCheckBox)checkBoxField.get(Itlum.this)).setSelected(false);
                        else
                        {
                            ((JCheckBox)checkBoxField.get(Itlum.this)).setSelected(true);
                            ((JSlider)sliderField.get(Itlum.this)).setValue(8-optionalRadioGroup.getActiveRadio().getLedButtonCouple().getY());
                        }
                    }
                    catch(Exception e)
                    {
                        e.printStackTrace();
                    }
                }
            });
            MidiNote baseNote;//Calculation of the Midi note to play
            if(i%2==0)
                baseNote=new MidiNote(Note.F_SHARP,1+i/2);
            else
                baseNote=new MidiNote(Note.C_SHARP,2+i/2);
            notePlayerListeners[i]=new NotePlayerListener<SpecificSequencerGroup,OptionalRadioGroup>(ScaleFactory.createMinorPentatonicScale(),baseNote,specificSequencerGroup,PlayerMode.PLAY_WITH_STEP_NUMBER)//Creation of the note player of the current internal sequencer
            {
                public void playNote(MidiNote midiNote)
                {
                    try
                    {
                        ShortMessage shortMessage=new ShortMessage();
                        shortMessage.setMessage(ShortMessage.NOTE_ON,0,midiNote.getMidiNoteNumber(),93);
                        synchronized(lock)
                        {
                            if(midiReady&&midiDevice!=null&&midiDevice.isOpen())
                                midiDevice.getReceiver().send(shortMessage,System.currentTimeMillis());
                        }
                        Thread.sleep(50);
                        shortMessage.setMessage(ShortMessage.NOTE_OFF,0,midiNote.getMidiNoteNumber(),0);
                        synchronized(lock)
                        {
                            if(midiReady&&midiDevice!=null&&midiDevice.isOpen())
                                midiDevice.getReceiver().send(shortMessage,System.currentTimeMillis());
                        }
                    }
                    catch(Exception e)
                    {
                        e.printStackTrace();
                    }
                }
            };
            specificSequencerGroup.addSequencerListener(notePlayerListeners[i]);//We add the note player to the current internal sequencer
        }
        monome.refresh();//Final refresh to initialize all leds

        //Finishing preparing the IHM
        System.out.println("Looking for connectable Midi devices");
        DefaultComboBoxModel midiDeviceComboBoxModel=new DefaultComboBoxModel();
        midiDeviceComboBox.setModel(midiDeviceComboBoxModel);
        Info[] infos=MidiSystem.getMidiDeviceInfo();
        for(Info info:infos)//We put all connectable Midi devices into the combo box
            try
            {
                midiDevice=MidiSystem.getMidiDevice(info);
                midiDevice.open();
                midiDevice.getReceiver();//Test of the device
                midiDeviceComboBoxModel.addElement(info);
            }
            catch(MidiUnavailableException e)
            {
            }
            finally
            {
                midiDevice.close();
            }
        try
        {
            midiDeviceComboBox.setSelectedIndex(0);
        }
        catch(IllegalArgumentException e)
        {
        }
        System.out.println("Creating all sequence iterators");
        DefaultComboBoxModel sequenceIteratorComboBoxModel=new DefaultComboBoxModel();
        sequenceIteratorComboBox.setModel(sequenceIteratorComboBoxModel);
        sequenceIteratorComboBoxModel.addElement(UpwardSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(BackwardSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(UpwardAndBackwardSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(SwingSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(TwiceUpwardSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(RandomOffsetSequenceIterator.class);
        sequenceIteratorComboBoxModel.addElement(RandomSequenceIterator.class);//We put all available sequence iterators into the combo box
        sequenceIteratorComboBox.setSelectedIndex(0);
        sequenceIteratorComboBox.setRenderer(new DefaultListCellRenderer()
        {
            @Override
            public Component getListCellRendererComponent(JList list,Object value,int index,boolean isSelected,boolean cellHasFocus)
            {
                JLabel label=(JLabel)super.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);
                label.setText(((Class<?>)value).getSimpleName());
                return label;
            }
        });
        System.out.println("Creating all scales");
        DefaultComboBoxModel scaleComboBoxModel=new DefaultComboBoxModel();
        scaleComboBox.setModel(scaleComboBoxModel);
        for(Scale scale:ScaleFactory.getAllScales(ScaleFactory.NAME_COMPARATOR))
            scaleComboBoxModel.addElement(scale);//We put all available musical scales into the combo box
        scaleComboBox.setSelectedIndex(12);

        //Making it visible
        System.out.println("Opening the main window");
        setVisible(true);
        System.out.println("Enjoy !");
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        sequencerPanel = new javax.swing.JPanel();
        checkBox1 = new javax.swing.JCheckBox();
        checkBox2 = new javax.swing.JCheckBox();
        checkBox3 = new javax.swing.JCheckBox();
        checkBox4 = new javax.swing.JCheckBox();
        checkBox5 = new javax.swing.JCheckBox();
        checkBox6 = new javax.swing.JCheckBox();
        checkBox7 = new javax.swing.JCheckBox();
        checkBox8 = new javax.swing.JCheckBox();
        slider1 = new javax.swing.JSlider();
        slider2 = new javax.swing.JSlider();
        slider3 = new javax.swing.JSlider();
        slider4 = new javax.swing.JSlider();
        slider5 = new javax.swing.JSlider();
        slider6 = new javax.swing.JSlider();
        slider7 = new javax.swing.JSlider();
        slider8 = new javax.swing.JSlider();
        scaleComboBox = new javax.swing.JComboBox();
        sequenceIteratorComboBox = new javax.swing.JComboBox();
        midiDevicePanel = new javax.swing.JPanel();
        midiDeviceComboBox = new javax.swing.JComboBox();
        connectButton = new javax.swing.JButton();
        quitButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(380, 510));
        getContentPane().setLayout(null);

        sequencerPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Sequencers"));
        sequencerPanel.setLayout(null);

        checkBox1.setText("1");
        checkBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox1ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox1);
        checkBox1.setBounds(20, 220, 40, 20);

        checkBox2.setText("2");
        checkBox2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox2ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox2);
        checkBox2.setBounds(60, 220, 40, 20);

        checkBox3.setText("3");
        checkBox3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox3ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox3);
        checkBox3.setBounds(100, 220, 40, 20);

        checkBox4.setText("4");
        checkBox4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox4ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox4);
        checkBox4.setBounds(140, 220, 40, 20);

        checkBox5.setText("5");
        checkBox5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox5ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox5);
        checkBox5.setBounds(180, 220, 40, 20);

        checkBox6.setText("6");
        checkBox6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox6ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox6);
        checkBox6.setBounds(220, 220, 40, 20);

        checkBox7.setText("7");
        checkBox7.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox7ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox7);
        checkBox7.setBounds(260, 220, 40, 20);

        checkBox8.setText("8");
        checkBox8.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkBox8ActionPerformed(evt);
            }
        });
        sequencerPanel.add(checkBox8);
        checkBox8.setBounds(300, 220, 40, 20);

        slider1.setMajorTickSpacing(1);
        slider1.setMaximum(8);
        slider1.setMinimum(1);
        slider1.setMinorTickSpacing(1);
        slider1.setOrientation(javax.swing.JSlider.VERTICAL);
        slider1.setPaintLabels(true);
        slider1.setPaintTicks(true);
        slider1.setSnapToTicks(true);
        slider1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider1StateChanged(evt);
            }
        });
        sequencerPanel.add(slider1);
        slider1.setBounds(10, 20, 40, 200);

        slider2.setMajorTickSpacing(1);
        slider2.setMaximum(8);
        slider2.setMinimum(1);
        slider2.setMinorTickSpacing(1);
        slider2.setOrientation(javax.swing.JSlider.VERTICAL);
        slider2.setPaintLabels(true);
        slider2.setPaintTicks(true);
        slider2.setSnapToTicks(true);
        slider2.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider2StateChanged(evt);
            }
        });
        sequencerPanel.add(slider2);
        slider2.setBounds(50, 20, 40, 200);

        slider3.setMajorTickSpacing(1);
        slider3.setMaximum(8);
        slider3.setMinimum(1);
        slider3.setMinorTickSpacing(1);
        slider3.setOrientation(javax.swing.JSlider.VERTICAL);
        slider3.setPaintLabels(true);
        slider3.setPaintTicks(true);
        slider3.setSnapToTicks(true);
        slider3.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider3StateChanged(evt);
            }
        });
        sequencerPanel.add(slider3);
        slider3.setBounds(90, 20, 40, 200);

        slider4.setMajorTickSpacing(1);
        slider4.setMaximum(8);
        slider4.setMinimum(1);
        slider4.setMinorTickSpacing(1);
        slider4.setOrientation(javax.swing.JSlider.VERTICAL);
        slider4.setPaintLabels(true);
        slider4.setPaintTicks(true);
        slider4.setSnapToTicks(true);
        slider4.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider4StateChanged(evt);
            }
        });
        sequencerPanel.add(slider4);
        slider4.setBounds(130, 20, 40, 200);

        slider5.setMajorTickSpacing(1);
        slider5.setMaximum(8);
        slider5.setMinimum(1);
        slider5.setMinorTickSpacing(1);
        slider5.setOrientation(javax.swing.JSlider.VERTICAL);
        slider5.setPaintLabels(true);
        slider5.setPaintTicks(true);
        slider5.setSnapToTicks(true);
        slider5.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider5StateChanged(evt);
            }
        });
        sequencerPanel.add(slider5);
        slider5.setBounds(170, 20, 40, 200);

        slider6.setMajorTickSpacing(1);
        slider6.setMaximum(8);
        slider6.setMinimum(1);
        slider6.setMinorTickSpacing(1);
        slider6.setOrientation(javax.swing.JSlider.VERTICAL);
        slider6.setPaintLabels(true);
        slider6.setPaintTicks(true);
        slider6.setSnapToTicks(true);
        slider6.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider6StateChanged(evt);
            }
        });
        sequencerPanel.add(slider6);
        slider6.setBounds(210, 20, 40, 200);

        slider7.setMajorTickSpacing(1);
        slider7.setMaximum(8);
        slider7.setMinimum(1);
        slider7.setMinorTickSpacing(1);
        slider7.setOrientation(javax.swing.JSlider.VERTICAL);
        slider7.setPaintLabels(true);
        slider7.setPaintTicks(true);
        slider7.setSnapToTicks(true);
        slider7.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider7StateChanged(evt);
            }
        });
        sequencerPanel.add(slider7);
        slider7.setBounds(250, 20, 40, 200);

        slider8.setMajorTickSpacing(1);
        slider8.setMaximum(8);
        slider8.setMinimum(1);
        slider8.setMinorTickSpacing(1);
        slider8.setOrientation(javax.swing.JSlider.VERTICAL);
        slider8.setPaintLabels(true);
        slider8.setPaintTicks(true);
        slider8.setSnapToTicks(true);
        slider8.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                slider8StateChanged(evt);
            }
        });
        sequencerPanel.add(slider8);
        slider8.setBounds(290, 20, 40, 200);

        scaleComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scaleComboBoxActionPerformed(evt);
            }
        });
        sequencerPanel.add(scaleComboBox);
        scaleComboBox.setBounds(20, 250, 310, 20);

        sequenceIteratorComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sequenceIteratorComboBoxActionPerformed(evt);
            }
        });
        sequencerPanel.add(sequenceIteratorComboBox);
        sequenceIteratorComboBox.setBounds(20, 280, 310, 20);

        getContentPane().add(sequencerPanel);
        sequencerPanel.setBounds(10, 10, 350, 320);

        midiDevicePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Midi device"));
        midiDevicePanel.setLayout(null);

        midiDevicePanel.add(midiDeviceComboBox);
        midiDeviceComboBox.setBounds(20, 30, 310, 20);

        connectButton.setText("Connect");
        connectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectButtonActionPerformed(evt);
            }
        });
        midiDevicePanel.add(connectButton);
        connectButton.setBounds(130, 60, 90, 23);

        getContentPane().add(midiDevicePanel);
        midiDevicePanel.setBounds(10, 330, 350, 100);

        quitButton.setText("Quit");
        quitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitButtonActionPerformed(evt);
            }
        });
        getContentPane().add(quitButton);
        quitButton.setBounds(140, 440, 90, 23);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    @SuppressWarnings("unchecked")
    private void sequenceIteratorComboBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_sequenceIteratorComboBoxActionPerformed
    {//GEN-HEADEREND:event_sequenceIteratorComboBoxActionPerformed
        try
        {
            GenericSequenceIteratorFactory<SpecificSequencerGroup,OptionalRadioGroup> genericSequenceIteratorFactory=GenericSequenceIteratorFactory.<SpecificSequencerGroup,OptionalRadioGroup>createGenericSequenceIteratorFactory((Class<? extends SequenceIterator>)sequenceIteratorComboBox.getSelectedItem());
            for(SpecificSequencerGroup specificSequencerGroup:multiSequencerGroup.getSequencers())
                specificSequencerGroup.setSequenceIterator(genericSequenceIteratorFactory.createSequenceIterator());
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_sequenceIteratorComboBoxActionPerformed

    private void scaleComboBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_scaleComboBoxActionPerformed
    {//GEN-HEADEREND:event_scaleComboBoxActionPerformed
        for(NotePlayerListener<SpecificSequencerGroup,OptionalRadioGroup> notePlayerListener:notePlayerListeners)
            notePlayerListener.setScale((Scale)scaleComboBox.getSelectedItem());
    }//GEN-LAST:event_scaleComboBoxActionPerformed

    private void quitButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_quitButtonActionPerformed
    {//GEN-HEADEREND:event_quitButtonActionPerformed
        if(midiDevice!=null)
            midiDevice.close();
        System.exit(0);
    }//GEN-LAST:event_quitButtonActionPerformed

    private void checkBox1ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox1ActionPerformed
    {//GEN-HEADEREND:event_checkBox1ActionPerformed
        try
        {
            if(checkBox1.isSelected())
            {
                int newStepsNumber=slider1.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(0).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(0).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(0).getLedButtonCouples()[8-slider1.getValue()].doClick();
                multiSequencerGroup.getSequencer(0).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(0).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox1ActionPerformed

    private void checkBox2ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox2ActionPerformed
    {//GEN-HEADEREND:event_checkBox2ActionPerformed
        try
        {
            if(checkBox2.isSelected())
            {
                int newStepsNumber=slider2.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(1).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(1).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(1).getLedButtonCouples()[8-slider2.getValue()].doClick();
                multiSequencerGroup.getSequencer(1).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(1).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox2ActionPerformed

    private void checkBox3ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox3ActionPerformed
    {//GEN-HEADEREND:event_checkBox3ActionPerformed
        try
        {
            if(checkBox3.isSelected())
            {
                int newStepsNumber=slider3.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(2).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(2).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(2).getLedButtonCouples()[8-slider3.getValue()].doClick();
                multiSequencerGroup.getSequencer(2).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(2).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox3ActionPerformed

    private void checkBox4ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox4ActionPerformed
    {//GEN-HEADEREND:event_checkBox4ActionPerformed
        try
        {
            if(checkBox4.isSelected())
            {
                int newStepsNumber=slider4.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(3).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(3).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(3).getLedButtonCouples()[8-slider4.getValue()].doClick();
                multiSequencerGroup.getSequencer(3).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(3).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox4ActionPerformed

    private void checkBox5ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox5ActionPerformed
    {//GEN-HEADEREND:event_checkBox5ActionPerformed
        try
        {
            if(checkBox5.isSelected())
            {
                int newStepsNumber=slider5.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(4).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(4).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(4).getLedButtonCouples()[8-slider5.getValue()].doClick();
                multiSequencerGroup.getSequencer(4).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(4).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox5ActionPerformed

    private void checkBox6ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox6ActionPerformed
    {//GEN-HEADEREND:event_checkBox6ActionPerformed
        try
        {
            if(checkBox6.isSelected())
            {
                int newStepsNumber=slider6.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(5).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(5).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(5).getLedButtonCouples()[8-slider6.getValue()].doClick();
                multiSequencerGroup.getSequencer(5).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(5).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox6ActionPerformed

    private void checkBox7ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox7ActionPerformed
    {//GEN-HEADEREND:event_checkBox7ActionPerformed
        try
        {
            if(checkBox7.isSelected())
            {
                int newStepsNumber=slider7.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(6).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(6).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(6).getLedButtonCouples()[8-slider7.getValue()].doClick();
                multiSequencerGroup.getSequencer(6).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(6).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox7ActionPerformed

    private void checkBox8ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_checkBox8ActionPerformed
    {//GEN-HEADEREND:event_checkBox8ActionPerformed
        try
        {
            if(checkBox8.isSelected())
            {
                int newStepsNumber=slider8.getValue();
                if(newStepsNumber!=multiSequencerGroup.getSequencer(7).getStepsNumberInSequence()||multiSequencerGroup.getSequencer(7).getButtonManager(0).getActiveRadio()==null)
                    multiSequencerGroup.getSequencer(7).getLedButtonCouples()[8-slider8.getValue()].doClick();
                multiSequencerGroup.getSequencer(7).setActive(true);
            }
            else
                multiSequencerGroup.getSequencer(7).setActive(false);
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_checkBox8ActionPerformed

    private void slider1StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider1StateChanged
    {//GEN-HEADEREND:event_slider1StateChanged
        try
        {
            if(checkBox1.isSelected()&&8-multiSequencerGroup.getSequencer(0).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider1.getValue())
                multiSequencerGroup.getSequencer(0).getLedButtonCouples()[8-slider1.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider1StateChanged

    private void slider2StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider2StateChanged
    {//GEN-HEADEREND:event_slider2StateChanged
        try
        {
            if(checkBox2.isSelected()&&8-multiSequencerGroup.getSequencer(1).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider2.getValue())
                multiSequencerGroup.getSequencer(1).getLedButtonCouples()[8-slider2.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider2StateChanged

    private void slider3StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider3StateChanged
    {//GEN-HEADEREND:event_slider3StateChanged
        try
        {
            if(checkBox3.isSelected()&&8-multiSequencerGroup.getSequencer(2).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider3.getValue())
                multiSequencerGroup.getSequencer(2).getLedButtonCouples()[8-slider3.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider3StateChanged

    private void slider4StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider4StateChanged
    {//GEN-HEADEREND:event_slider4StateChanged
        try
        {
            if(checkBox4.isSelected()&&8-multiSequencerGroup.getSequencer(3).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider4.getValue())
                multiSequencerGroup.getSequencer(3).getLedButtonCouples()[8-slider4.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider4StateChanged

    private void slider5StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider5StateChanged
    {//GEN-HEADEREND:event_slider5StateChanged
        try
        {
            if(checkBox5.isSelected()&&8-multiSequencerGroup.getSequencer(4).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider5.getValue())
                multiSequencerGroup.getSequencer(4).getLedButtonCouples()[8-slider5.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider5StateChanged

    private void slider6StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider6StateChanged
    {//GEN-HEADEREND:event_slider6StateChanged
        try
        {
            if(checkBox6.isSelected()&&8-multiSequencerGroup.getSequencer(5).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider6.getValue())
                multiSequencerGroup.getSequencer(5).getLedButtonCouples()[8-slider6.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider6StateChanged

    private void slider7StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider7StateChanged
    {//GEN-HEADEREND:event_slider7StateChanged
        try
        {
            if(checkBox7.isSelected()&&8-multiSequencerGroup.getSequencer(6).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider7.getValue())
                multiSequencerGroup.getSequencer(6).getLedButtonCouples()[8-slider7.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider7StateChanged

    private void slider8StateChanged(javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_slider8StateChanged
    {//GEN-HEADEREND:event_slider8StateChanged
        try
        {
            if(checkBox8.isSelected()&&8-multiSequencerGroup.getSequencer(7).getButtonManager(0).getActiveRadio().getLedButtonCouple().getY()!=slider8.getValue())
                multiSequencerGroup.getSequencer(7).getLedButtonCouples()[8-slider8.getValue()].doClick();
        }
        catch(MonomeException e)
        {
            e.printStackTrace();
        }
    }//GEN-LAST:event_slider8StateChanged

    private void connectButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_connectButtonActionPerformed
    {//GEN-HEADEREND:event_connectButtonActionPerformed
        synchronized(lock)
        {
            if(connectButton.getText().equals("Connect"))
            {
                try
                {
                    midiDeviceComboBox.setEnabled(false);
                    midiDevice=MidiSystem.getMidiDevice((Info)midiDeviceComboBox.getSelectedItem());
                    midiDevice.open();
                    midiDevice.getReceiver();//Test of the device
                    connectButton.setText("Disconnect");
                    midiReady=true;
                }
                catch(MidiUnavailableException e)
                {
                    e.printStackTrace();
                    midiDeviceComboBox.setEnabled(true);
                }
            }
            else
            {
                midiReady=false;
                if(midiDevice!=null)
                    midiDevice.close();
                midiDevice=null;
                connectButton.setText("Connect");
                midiDeviceComboBox.setEnabled(true);
            }
        }
    }//GEN-LAST:event_connectButtonActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[])
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                try
                {
                    new Itlum();
                }
                catch(Exception e)
                {
                    e.printStackTrace();
                }
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JCheckBox checkBox1;
    public javax.swing.JCheckBox checkBox2;
    public javax.swing.JCheckBox checkBox3;
    public javax.swing.JCheckBox checkBox4;
    public javax.swing.JCheckBox checkBox5;
    public javax.swing.JCheckBox checkBox6;
    public javax.swing.JCheckBox checkBox7;
    public javax.swing.JCheckBox checkBox8;
    private javax.swing.JButton connectButton;
    private javax.swing.JComboBox midiDeviceComboBox;
    private javax.swing.JPanel midiDevicePanel;
    private javax.swing.JButton quitButton;
    private javax.swing.JComboBox scaleComboBox;
    private javax.swing.JComboBox sequenceIteratorComboBox;
    private javax.swing.JPanel sequencerPanel;
    public javax.swing.JSlider slider1;
    public javax.swing.JSlider slider2;
    public javax.swing.JSlider slider3;
    public javax.swing.JSlider slider4;
    public javax.swing.JSlider slider5;
    public javax.swing.JSlider slider6;
    public javax.swing.JSlider slider7;
    public javax.swing.JSlider slider8;
    // End of variables declaration//GEN-END:variables

}
